/*-
 * Copyright 2010 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __WINTV_HVR1850_H__
#define __WINTV_HVR1850_H__

#include <sys/errno.h>
#include "tda18271.h"
#include "s5h1411.h"
#include "cx88_debug.h"
#include "cx88_mpeg_driver.h"

class wintv_hvr1850
   : public cx88_mpeg_driver
{
   public:
      
      wintv_hvr1850(
         cx88_xml_config &cx88conf,
         tuner_config &tunerconf,
         const char *devnode,
         capvec &captures,
         int &error)
         : cx88_mpeg_driver(cx88conf, tunerconf, devnode, captures, error)
      {}
      
      virtual ~wintv_hvr1850(void) {}
      
   protected:

      static int tda18271_interface_config(tda18271 &driver, const dvb_channel &channel, tda18271_interface &ifc)
      {
         ifc.std = 0x0;
         ifc.if_level = 0x06;
         switch (channel.modulation)
         {
            case DVB_MOD_VSB_8:
            case DVB_MOD_VSB_16:
               ifc.std = 0x03;
               ifc.ifreq_hz = 5380000;
               break;
            default:
               break;
         }
         return 0;
      }

      virtual int create_i2c_drivers(void)
      {
         int error = 0;
         cx88_common_gpio gpio;
         gpio.in_write_mask = 0x6000;
         gpio.in_read_mask = 0x0;
         gpio.inout_gpio = 0x0;
         if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
         {
            perror("Error during microcontroller GPIO ioctl");
            return errno;
         }
         usleep(100000);
         gpio.inout_gpio = 0x4000;
         if (ioctl(m_fd, CX23885_COMMON_MC_GPIO, &gpio) != 0)
         {
            perror("Error during microcontroller GPIO ioctl");
            return errno;
         }
         usleep(100000);
         m_i2c_devs[1].driver = new s5h1411(m_tunerconf, *(m_i2c_devs[1].dev), *(m_i2c_devs[2].dev), DVB_INPUT_SERIAL, 
            S5H1411_IFREQ_5_38MHZ, S5H1411_IFREQ_4MHZ, S5H1411_GPIO_DISABLE, S5H1411_CLK_CONT_NINV, error);
         m_i2c_devs[0].driver = new tda18271(m_tunerconf, *(m_i2c_devs[0].dev), TDA18271_MODE_SINGLE,
               NULL, tda18271_interface_config, error);
         if ((m_i2c_devs[0].driver == NULL) || (m_i2c_devs[1].driver == NULL))
         {
            error = ENOMEM;
         }
         if (error)
         {
            CX88ERR << "Error initializing tuner/demodulator driver for WinTV HVR-1850" << std::endl;
            return error;
         }
         return 0;
      }
      
      virtual void destroy_i2c_drivers(void)
      {
         delete m_i2c_devs[0].driver;
         delete m_i2c_devs[1].driver;  
      }
      
      virtual int create_i2c_devices(void)
      {
         m_i2c_devs[0].conf.in_addr = 0x60;
         // The TDA18271 on the 1850 will NAK 400KHz transactions.
         // This is odd, since the datasheet indicates it supports
         // 400KHz, and the earlier-revision TDA18271 on
         // the 1800 seems fine w/ 400KHz.
         //m_i2c_devs[0].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[1].conf.in_addr = 0x19;
         m_i2c_devs[1].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         m_i2c_devs[2].conf.in_addr = 0x1A;
         m_i2c_devs[2].conf.in_speed = CX88_I2C_SPEED_400KHZ;
         return 0;
      }
      
      virtual int set_capture_options(cx88_mpeg_capture &capture)
      {
         capture.in_start_byte_count = 0;
         return 0;
      }
      
      virtual int set_channel_options(dvb_channel &channel) 
      {
         if (channel.inversion == DVB_INVERSION_AUTO)
         {
            channel.inversion = DVB_INVERSION_ON;
         }
         return 0;
      }

};

#endif
